/**
 * eip-addrcache.cc
 */
#ifndef __EIP_ADDRCACHE_CC__
#define __EIP_ADDRCACHE_CC__

#include <stdlib.h>
#include <stdio.h>
#include "./eip-addrcache.h"
#include "./memdmp.h"
#include "./eip-constants.h"
#include "./uthash.h"    // macro-based hash table implementation

const int debug_eip_addrcache = 1;          //DEBUG FLAG

// *** global pointer to the hash table that stores the addr cache ***
ADDR_CACHE_RECORD *addrcache = NULL;

void addrcache_add(TADDRESS* taddr){
	addrcache_add_(taddr, 0);
}

void addrcache_add_sent(TADDRESS* taddr){
	addrcache_add_(taddr, 1);
}

void addrcache_add_(TADDRESS* taddr, int sent){
  ADDR_CACHE_RECORD *rec;
  rec = addrcache_lookup((ADDR_CACHE_KEY*)taddr);
  if(rec != NULL){
	  addrcache_del_internal(rec);
  }
  // create and zero new record
  rec = (ADDR_CACHE_RECORD*)malloc(sizeof(ADDR_CACHE_RECORD));
  memset(rec, 0, sizeof(ADDR_CACHE_RECORD));
  // copy data to new record
  if(debug_eip_addrcache){ printf("...caching addr for ");memdmp((char*)taddr->src, EIP_NET_ADDR_LEN*2);printf("\n"); }
  memcpy(&rec->key, taddr->src, EIP_NET_ADDR_LEN*2); // copy dstt src+dst net addr to key
  memcpy(&rec->taddr, taddr, sizeof(TADDRESS));   // copy t-address  
  rec->sent = sent; // set flag indicating this send status of this address
  // insert new record into hash
  HASH_ADD(hh, addrcache, key, sizeof(ADDR_CACHE_KEY), rec);
}


void addrcache_del(ADDR_CACHE_RECORD *rec){
	addrcache_del_(rec, 0);
}

void addrcache_del_internal(ADDR_CACHE_RECORD *rec){
	addrcache_del_(rec, 1);
}

void addrcache_del_(ADDR_CACHE_RECORD *rec, int internal){
  if(debug_eip_addrcache){
	  printf("...dropping addr for ");memdmp((char*)rec, EIP_NET_ADDR_LEN*2);
	  printf((internal)?" (internal)": " (external)");
	  printf("\n"); 
  }
  HASH_DELETE(hh, addrcache, rec);
  free(rec);
}

ADDR_CACHE_RECORD* addrcache_lookup(ADDR_CACHE_KEY* needle){
  ADDR_CACHE_RECORD *rec;
  HASH_FIND(hh, addrcache, needle, sizeof(ADDR_CACHE_KEY), rec);
  if(debug_eip_addrcache){
	  printf("...addr lookup for key: ");memdmp((char*)needle, EIP_NET_ADDR_LEN*2);
	  printf((rec == NULL)?" FAIL":" OK");
	  printf("\n");
  }// debug
  return rec;
}

ADDR_CACHE_RECORD* addrcache_lookup_reverse(ADDR_CACHE_KEY* needle){
	ADDR_CACHE_KEY swapper;
	memcpy(&swapper, needle, sizeof(ADDR_CACHE_KEY));
	int tmp = swapper.src;
	swapper.src = swapper.dst;
	swapper.dst = tmp;
	return addrcache_lookup(&swapper);
}

#endif // __EIP_ADDRCACHE_CC__
