/**
 * eip-addrcache.h
 */

#ifndef __EIP_ADDRCACHE_H__
#define __EIP_ADDRCACHE_H__

#include <stdlib.h>
#include <stdio.h>
#include "./memdmp.h"
#include "./eip-constants.h"
#include "./uthash.h"    // macro-based hash table implementation
 
#define ADDR_CACHE_KEY struct eip_addr_cache_key
ADDR_CACHE_KEY { // this structure will be our hash key
  int src; // network address of the source
  int dst; // network address of the destination
};

#define ADDR_CACHE_RECORD struct eip_addr_record
ADDR_CACHE_RECORD {
  ADDR_CACHE_KEY key; // hash key
  TADDRESS taddr;     // t-address stored in this record
  uint8_t sent;       // flag indicating whether this record has been sent
  UT_hash_handle hh;  // makes the structure hashable
};

/**
 * add a record to the addr cache, with sent flag = 0
 *  - takes a memory buffer as argument (must be a t-address)
 */
void addrcache_add(TADDRESS *taddr);

/**
 * add a record to the addr cache, with sent flag = 1
 *  - takes a memory buffer as argument (must be a t-address)
 */
void addrcache_add_sent(TADDRESS *taddr);


/**
 * add a record to the addr cache
 *  - takes a memory buffer as argument (must be a t-address)
 */
void addrcache_add_(TADDRESS *taddr, int sent);


/**
 * delete a record from the cache (by pointer), from external source
 * 
 */ 
void addrcache_del(ADDR_CACHE_RECORD *rec);

/**
 * delete a record from the cache (by pointer), from internal source
 * 
 */ 
void addrcache_del_internal(ADDR_CACHE_RECORD *rec);

/**
 * delete a record from the cache (by pointer), with flag indicating whether call came from inside the API
 * 
 */ 
void addrcache_del_(ADDR_CACHE_RECORD *rec, int internal);

/**
 * Returns a pointer to a addr cache record based on a lookup key
 * 
 */ 
ADDR_CACHE_RECORD* addrcache_lookup(ADDR_CACHE_KEY* needle);

/**
 * Returns a pointer to a addr cache record based on a lookup key with parameters reversed (i.e., swap src/dst)
 * 
 */ 
ADDR_CACHE_RECORD* addrcache_lookup_reverse(ADDR_CACHE_KEY* needle);


#endif // __EIP_ADDRCACHE_H__
