/*
 * File: eip_timer.h
 * Description: 
 *   A timer for single-threaded processing of EIP datagrams.
 *   This timer requires that processing time in nanoseconds
 *   fit into the number of bytes for an unsigned int.
 */
 #ifndef __EIP_TIMER_H__
 #define __EIP_TIMER_H__

// Uncomment the next line to disable all router performance assessment
// #define EIP_DISABLE_ROUTER_PERF_ASSESS
 
// Uncomment the next two lines to enable PAPI cycle counting
// #define EIP_PAPI_CYCLE_COUNTING
// #include <papi.h>


// Uncomment the next line to enable Intel assembly cycle counting
// #define EIP_ASM_CYCLE_COUNTING
 
 void eip_timer_start();
 void eip_timer_stop(unsigned int* elapsed_nanoseconds);
 
#ifdef EIP_ASM_CYCLE_COUNTING
 /*
  * Functions taken from CMU systems programming book.  These access
  * the instruction counter from the IA32 instruction set, so this
  * code will not build on systems that do not support the RTDSC instruction
  */
void eip_asm_access_counter(unsigned *hi, unsigned *lo);
void eip_asm_start_counter();
double eip_asm_get_counter();
#endif // EIP_ASM_CYCLE_COUNTING
 
#endif // __EIP_TIMER_H__
