/**
 * File:
 *   eip-signcache.h
 * Description:
 *   Signed taddr cache implementation using uthash
 *   Keys are a concatentation of src ip addr and dst ip addr
 * Author:
 *   ebj8
 * History:
 *   2012 12 28 ebj8: created from flowcache.h -- please excuse the copy-pasta for now
 */
#ifndef __EIP_SIGNCACHE_H__
#define __EIP_SIGNCACHE_H__

#include <stdlib.h>
#include <stdio.h>
#include <netinet/ip.h>
#include "../common/memdmp.h"
#include "../common/eip-constants.h"
#include "../common/uthash.h"    // macro-based hash table implementation

#define SIGN_CACHE_KEY struct eip_sign_cache_key
SIGN_CACHE_KEY { // this structure will be our hash key
  int src; // network address of the source
  int dst; // network address of the destination
};

#define SIGN_CACHE_RECORD struct eip_sign_record
SIGN_CACHE_RECORD {
  SIGN_CACHE_KEY key; // hash key
  TADDRESS taddr;     // t-address for the destination of this flow
  int ready;                // whether this signature is ready
  UT_hash_handle hh;  // makes the structure hashable
};

/**
 * Thread execution function
 */ 
void *signcache_run(void *arg);

/**
 * get a signature for a src/dst combo
 */ 
int signcache_get(iphdr *ip, TADDRESS *taddr);

/**
 * insert a record for a src/dst combo
 */ 
void signcache_insert(iphdr* ip, TADDRESS* taddr);

/**
 *  sweep cache looking for unready signatures
 */
void signcache_sweep();

/**
 *  sign a taddress
 */
void signcache_sign(TADDRESS* taddr);

/**
 * setup the sign cache:
 *    - initialize mutex
 *    - start thread
 *    
 */
void signcache_setup();




///////////////////////////////////////////////////////

/**
 * add a record to the flow cache
 *  - takes a memory buffer as argument (must be eip type 1 datagram)
 */
void signcache_add(EIPTYPE1* e1);

void signcache_update(SIGN_CACHE_RECORD* rec, EIPTYPE3 *e3);

/**
 * delete a record from the cache (by pointer)
 * 
 */ 
void signcache_del(SIGN_CACHE_RECORD *rec);

/**
 * Returns a pointer to a flow cache record based on a lookup key
 * 
 */ 
SIGN_CACHE_RECORD* signcache_lookup(SIGN_CACHE_KEY* needle);

/**
 * Returns a pointer to a flow cache record based on a lookup key with the components reversed (swap src/dst)
 * 
 */ 
SIGN_CACHE_RECORD* signcache_lookup_reverse(SIGN_CACHE_KEY* needle);


/**
 * Returns a pointer to a flow cache record based on a flow id
 * 
 */ 
SIGN_CACHE_RECORD* signcache_find(char* signid);


#endif // ifndef __EIP_FLOWCACHE_H__
