/*
 * File: eip_router.h
 */
#ifndef __EIP_ROUTER_H__
#define __EIP_ROUTER_H__

#include <stdlib.h> // for malloc
#include <stdint.h> // for uint types
#include <stdio.h>  // for printf
#include <string.h> // for memcpy
#include <arpa/inet.h>   // for htons
#include <netinet/ip.h>  // added for struct iphdr
#include <time.h> // for time
#include "../common/eip-routercache.h"
#include "../common/eip-constants.h"
#include "../common/netfilter-constants.h"

#define EIP_ROUTER_VERIFICATIONS 2  // number of RSA verifications to execute for each verification action
int eip_flow_verify(const unsigned char* eip_hdr);
int eip_router_validate(const unsigned char *pkt, const unsigned int len);
int eip_validate_type1(const unsigned char* eip, const unsigned char* ip);
int eip_validate_type2(const unsigned char* eip, const unsigned char* ip);
int eip_validate_type3(const unsigned char* eip, const unsigned char* ip, const int hasContinuation);
int eip_router_verify(char* data, unsigned int datalen, char* sig, unsigned int siglen, char* pubkey, int pubkeylen);

/*
 * Writes processing time to packet and returns given return value.
 * Meant to be final step in processing before returning verdict to netfilter queue service.
 */
int eip_router_return(int datagram_type, int continuation_flag, int return_value, unsigned char* eip);

#endif // __EIP_ROUTER_H__
