/**
 * EIP router netfilter queue service for the router
 */
 

using namespace std;
#include <iostream>
#include <iomanip>
#include <stdlib.h> // added to get exit() to work
#include <string.h> // added for memcpy
#include <time.h>
#include <netinet/in.h>
#include <netinet/ip.h>  // added for struct iphdr
#include <netinet/tcp.h> // added for struct tcphdr
#include <arpa/inet.h>   // for htons
extern "C" {
  #include <linux/netfilter.h>  /* Defines verdicts (NF_ACCEPT, etc) */
  #include <libnetfilter_queue/libnetfilter_queue.h>
}
// crypto includes
#include <openssl/bio.h>
#include <openssl/evp.h>
#include <openssl/pem.h>
#include <openssl/rsa.h>
#include <string.h>

extern EVP_MD_CTX *eip_evp_md_ctx; // from eip-rsa-sign-verify
extern uint64_t now;                 // from eip_router

// EIP common includes
#include "../common/netfilter-constants.h" // netfilter common values
#include "../common/memdmp.h" // for memdmp debugging function

#include "eip_router.h"

// globals for callback function
nfqnl_msg_packet_hdr *nfCallbackPacketHeader = NULL;
uint32_t nfCallbackPacketId = 0;
int nfCallbackVerdict = 0;
unsigned int nfCallbackPacketBytes = 0;
unsigned char *nfCallbackPacketData = NULL;
int nfCallbackDebug = 0; // debug flag

/**
 * callback function for netfilter hook
 */
 static int Callback(nfq_q_handle *myQueue, struct nfgenmsg *msg, nfq_data *pkt, void *cbData) {
	now = (uint64_t)time(NULL);
    if ((nfCallbackPacketHeader = nfq_get_msg_packet_hdr(pkt))) {
        nfCallbackPacketId = ntohl(nfCallbackPacketHeader->packet_id);	 
    }
    nfCallbackPacketBytes = nfq_get_payload(pkt, &nfCallbackPacketData); // get pointer to packet data and # of bytes in packet
    if(nfCallbackDebug){
        printf("DEBUG: packet length=%d header=", nfCallbackPacketBytes);
        memdmp((char*)nfCallbackPacketData, 20);
        printf("\n");
    }
    if(eip_router_validate(nfCallbackPacketData, nfCallbackPacketBytes)){
		nfCallbackVerdict = nfq_set_verdict(myQueue, nfCallbackPacketId, NF_ACCEPT, 0, NULL); // return original packet    
	} else {
		nfCallbackVerdict = nfq_set_verdict(myQueue, nfCallbackPacketId, NF_DROP, 0, NULL); // drop invalid packet    
	}
    return nfCallbackVerdict;
}

 /**
  * main (loop) 
  */
int main(int argc, char **argv) {
    printf("EIP Router started -- press Control-C to stop\n");
    if(argc == 2){
        nfCallbackDebug = 1; // enable debugging based on command-line option
        printf("Debug mode enabled.\n");
    }
    srand(time(NULL)); // seed random number generator
    // crypto init
    OpenSSL_add_all_digests();  // load all known digests
    eip_evp_md_ctx = EVP_MD_CTX_create();  // get a new envelope message digest context
    EVP_MD_CTX_init(eip_evp_md_ctx);       // initialize envelope message digest context

    // router cache bitmap initialization
    routercache_init();

    struct nfq_handle *nfqHandle;
    struct nfq_q_handle *myQueue;
    struct nfnl_handle *netlinkHandle;
    int fd, res;
    char buf[NF_BUFFER_LEN]; // this is the main packet buffer
    // Get a queue connection handle from the module
    if (!(nfqHandle = nfq_open())) {
        cerr << "Error in nfq_open()" << endl;
        exit(-1);
    }
    // Unbind the handler from processing any IP packets
    if (nfq_unbind_pf(nfqHandle, AF_INET) < 0) {
        cerr << "Error in nfq_unbind_pf()" << endl;
        exit(1);
    }
    // Bind this handler to process IP packets...
    if (nfq_bind_pf(nfqHandle, AF_INET) < 0) {
         cerr << "Error in nfq_bind_pf()" << endl;
         exit(1);
    }
    // Install a callback on queue NF_QUEUE_NUM (queue number arbitrary, see notes above)
    if (!(myQueue = nfq_create_queue(nfqHandle,  NF_QUEUE_NUM, &Callback, NULL))) {
        cerr << "Error in nfq_create_queue()" << endl;
        exit(1);
    }
    // Turn on packet copy mode
    if (nfq_set_mode(myQueue, NFQNL_COPY_PACKET, 0xffff) < 0) {
    cerr << "Could not set packet copy mode" << endl;
    exit(1);
  }
  netlinkHandle = nfq_nfnlh(nfqHandle);
  fd = nfnl_fd(netlinkHandle);
  //while ((res = recv(fd, buf, sizeof(buf), 0)) && res >= 0) {
    while(true){
    res = recv(fd, buf, sizeof(buf), 0);
    if(res >= 0) nfq_handle_packet(nfqHandle, buf, res);
    //else cerr << "NFQUEUE_BUFFER_ERROR" << endl;//uncomment this line to see error messages
    // end while receiving traffic
  }
  nfq_destroy_queue(myQueue);
  nfq_close(nfqHandle);
  return 0;
  // end main
}
